/*  ------------------------------------------- */
/*  FFTW2IDL: Wrapper for using FFTW in IDL     */
/*  14.2.2002 by Andreas Reigber                */
/*  published under the GNU public licence (GPL)*/
/*  Email: anderl@nought.de                     */
/*  ------------------------------------------- */
/*  FFTW1D.C                                    */
/*  1D transformation over complex 1D Array     */
/*  ------------------------------------------- */
/*  Usage in IDL:                               */
/*  fftw1d_iniplan,arr,direc,plan               */
/*  fftw1d,arr,plan                             */
/*  fftw1d_delplan,plan                         */
/*  ------------------------------------------- */

#include "fftw.h"
#include "rfftw.h"
#include "export.h"
#include <stdlib.h>
#include <stdio.h>

void fftw1d(argc, argv)
int argc;
IDL_VPTR argv[];
{
	IDL_VPTR in_idl, plan_idl; 
	fftw_plan plan;
	fftw_complex *in;
	
	in_idl   = argv[0];
	plan_idl = argv[1];

	if (in_idl->type != IDL_TYP_COMPLEX)
	{
		printf("ERROR: Input array has to be complex (single precision)\n");
		return;
	}
	if (in_idl->value.arr->n_dim != 1)
	{
		printf("ERROR: Input array not one-dimensional\n");
		return;
	}

	in   = (fftw_complex *)in_idl->value.arr->data;
	plan = (fftw_plan)plan_idl->value.l;
	
	fftw_one(plan, in, NULL);
}


void fftw1d_iniplan(argc, argv)
int argc;
IDL_VPTR argv[];
{
	IDL_VPTR in_idl, direc_idl, plan_idl; 
	fftw_plan plan;
	FILE *input_file;
	FILE *output_file;
/*	fftw_complex *in;  */
	int direc;
	int anzx;
	
	in_idl    = argv[0];
	direc_idl = argv[1];
	plan_idl  = argv[2];
	direc = direc_idl->value.i;
	
	if (in_idl->type != IDL_TYP_COMPLEX)
	{
		printf("ERROR: Input array has to be complex (single precision)\n");
		return;
	}
	if (in_idl->value.arr->n_dim != 1)
	{
		printf("ERROR: Input array not one-dimensional\n");
		return;
	}
	if (direc != 1 && direc != -1 )
	{
		printf("ERROR: Direction argument has to be -1 (forward FFT) or 1 (backward FFT)\n");
		return;
	}
	
	anzx  = in_idl->value.arr->dim[0];	
/*	in    = (fftw_complex *)in_idl->value.arr->data; */

	input_file = fopen("fftw.wisdom", "r");
	fftw_import_wisdom_from_file(input_file);
	fclose(input_file);

	if (direc == -1) 
		plan = fftw_create_plan(anzx, FFTW_FORWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);
	else
		plan = fftw_create_plan(anzx, FFTW_BACKWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);

	output_file = fopen("fftw.wisdom", "w");                     
	fftw_export_wisdom_to_file(output_file);
	fclose(output_file);

	plan_idl->value.l = (IDL_LONG)plan;
}

void fftw1d_delplan(argc, argv)
int argc;
IDL_VPTR argv[];
{
	IDL_VPTR plan_idl; 
	fftw_plan plan;
	plan_idl = argv[0];
	plan = (fftw_plan)plan_idl->value.l;
	
	fftw_destroy_plan(plan);	
}

