/*  ------------------------------------------- */
/*  FFTW2IDL: Wrapper for using FFTW in IDL     */
/*  (c) by Andreas Reigber and Pau Prats        */
/*  published under the GNU public licence (GPL)*/
/*  Email: anderl@nought.de                     */
/*  ------------------------------------------- */
/*  FFTW1D_ARR.C (2 Processors multithreaded)   */
/*  1D transformation over complex 2D Array     */
/*  ------------------------------------------- */
/*  Usage in IDL:                               */
/*  fftw1d_arr,arr,dim,direc                    */
/*  ------------------------------------------- */

#include <stdlib.h>
#include <stdio.h>
#include "fftw_threads.h"
#include "rfftw_threads.h"
#include "export.h"
#include "windowsdll.h"

#ifdef WIN32
IDL_DLL_API 
#endif

void fftw1d_arr(argc, argv)
int argc;
IDL_VPTR argv[];
{
	IDL_VPTR in_idl, dim_idl, direc_idl; 
	fftw_plan plan;
	FILE *input_file;
	FILE *output_file;
	fftw_complex *in; 
	int dim;
	int direc;
	int anz, anzx, anzy;
	IDL_VPTR	 idlvptr;
	
	in_idl    = argv[0];
	dim_idl   = argv[1];
	direc_idl = argv[2];
	dim   = dim_idl->value.i;
	direc = direc_idl->value.i;

	if (in_idl->type != IDL_TYP_COMPLEX)
	{
		idlvptr = IDL_StrToSTRING("ERROR: Input array has to be complex (single precision)");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return;
	}
	if (in_idl->value.arr->n_dim != 2)
	{
		idlvptr = IDL_StrToSTRING("ERROR: Input array not two-dimensional");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return;
	}
	if (dim != 1 && dim != 2)
	{
		idlvptr = IDL_StrToSTRING("ERROR: Dimension argument has to be 1 or 2");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return;
	}
	if (direc != 1 && direc != -1 )
	{
		idlvptr = IDL_StrToSTRING("ERROR: Direction argument has to be -1 (forward FFT) or 1 (backward FFT)");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return;
	}
	
	anzx = in_idl->value.arr->dim[0];
	anzy = in_idl->value.arr->dim[1];
	if (dim == 1) anz = anzx; else anz = anzy;
	
	in  = (fftw_complex *)in_idl->value.arr->data; 
	
	fftw_threads_init();
	 
	input_file = fopen("fftw.wisdom", "r");
	if (input_file)
	{
		fftw_import_wisdom_from_file(input_file);
		fclose(input_file);
	}

	if (direc == -1)
		plan = fftw_create_plan(anz,FFTW_FORWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);
	else
		plan = fftw_create_plan(anz,FFTW_BACKWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);

	output_file = fopen("fftw.wisdom", "w");                     
	fftw_export_wisdom_to_file(output_file);
	fclose(output_file);
 
	if (dim == 1)
	{
		fftw_threads(2, plan, anzy, in, 1, anzx, NULL, 1, anzx);
	}
	else
	{ 
		fftw_threads(2, plan, anzx, in, anzx, 1, NULL, anzx, 1);
	}
	
	fftw_destroy_plan(plan);	
}

