/*  ------------------------------------------- */
/*  FFTW2IDL: Wrapper for using FFTW in IDL     */
/*  (c) by Andreas Reigber and Pau Prats        */
/*  published under the GNU public licence (GPL)*/
/*  Email: anderl@nought.de                     */
/*  ------------------------------------------- */
/*  FFTW2D_CR_T4.C (2 processor multithreaded)  */
/*  2D real to complex transformation           */
/*  ------------------------------------------- */
/*  Usage in IDL:                               */
/*  res=fftw2d_rc(arr)                          */
/*  ------------------------------------------- */


#include "fftw_threads.h"
#include "rfftw_threads.h"
#include "export.h"
#include <stdlib.h>
#include <stdio.h>
#include "windowsdll.h"

#ifdef WIN32
IDL_DLL_API 
#endif

IDL_VPTR fftw2d_rc(argc, argv)
int argc;
IDL_VPTR argv[];
{
	IDL_VPTR in_idl, out_idl; 
	fftwnd_plan plan;
	FILE *input_file;
	FILE *output_file;
	fftw_real *in;
	fftw_complex *out;
	int anzx, anzy;
	long cdim[2];
	IDL_VPTR	 idlvptr;

	in_idl    = argv[0];

	if (in_idl->type != IDL_TYP_FLOAT)
	{
		idlvptr = IDL_StrToSTRING("ERROR: Input array has to be float (single precision)");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return(IDL_GettmpInt(-1));
	}
	if (in_idl->value.arr->n_dim != 2)
	{
		idlvptr = IDL_StrToSTRING("ERROR: Input array not two-dimensional");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return(IDL_GettmpInt(-1));
	}

	anzx = in_idl->value.arr->dim[0];
	anzy = in_idl->value.arr->dim[1];
	cdim[0] = (long)(anzx/2 + 1);
	cdim[1] = (long)(anzy);
	
	in  = (fftw_real *)in_idl->value.arr->data;
		
	fftw_threads_init();
	  	 
	input_file = fopen("fftw.wisdom", "r");
	if (input_file)
	{
		fftw_import_wisdom_from_file(input_file);
		fclose(input_file);
	}

	plan = rfftw2d_create_plan(anzy, anzx, FFTW_REAL_TO_COMPLEX,FFTW_MEASURE | FFTW_USE_WISDOM);
	 
	output_file = fopen("fftw.wisdom", "w");                     
	fftw_export_wisdom_to_file(output_file);
	fclose(output_file);

	out = (fftw_complex *)IDL_MakeTempArray(IDL_TYP_COMPLEX,2,cdim,IDL_BARR_INI_NOP, &out_idl);
	
	rfftwnd_threads_one_real_to_complex(2, plan, in, out);

	rfftwnd_destroy_plan(plan);	
	
	return(out_idl);
}

