/*  ------------------------------------------- */
/*  FFTW2IDL: Wrapper for using FFTW in IDL     */
/*  (c) by Andreas Reigber and Pau Prats        */
/*  published under the GNU public licence (GPL)*/
/*  Email: anderl@nought.de                     */
/*  ------------------------------------------- */
/*  FFTW2D_T2.C    (4 processors multithreaded) */
/*  2D transformation over complex 2D Array     */
/*  ------------------------------------------- */

#include "fftw_threads.h"
#include "rfftw_threads.h"
#include "export.h"
#include <stdlib.h>
#include <stdio.h>
#include "windowsdll.h"

#ifdef WIN32
IDL_DLL_API 
#endif

void fftw2d(argc, argv)
int argc;
IDL_VPTR argv[];
{
	IDL_VPTR in_idl, direc_idl; 
	fftwnd_plan plan;
	char *wisdom_string;
	FILE *input_file;
	FILE *output_file;
	fftw_complex *in;
	int direc;
	int anzx, anzy;
	IDL_VPTR	 idlvptr;

	in_idl    = argv[0];
	direc_idl = argv[1];
	direc = direc_idl->value.i;
	
	if (in_idl->type != IDL_TYP_COMPLEX)
	{
		idlvptr = IDL_StrToSTRING("ERROR: Input array has to be float (single precision)");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return;
	}
	if (in_idl->value.arr->n_dim != 2)
	{
		idlvptr = IDL_StrToSTRING("ERROR: Input array not two-dimensional");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return;
	}
	if (direc != 1 && direc != -1 )
	{
		idlvptr = IDL_StrToSTRING("ERROR: Direction argument has to be -1 (forward FFT) or 1 (backward FFT)");
		IDL_Print(1,&idlvptr,0);
		IDL_DELTMP(idlvptr);
		return;
	}
	
	anzx = in_idl->value.arr->dim[0];
	anzy = in_idl->value.arr->dim[1];
	in  = (fftw_complex *)in_idl->value.arr->data;
	
	fftw_threads_init();
	 
	input_file = fopen("fftw.wisdom", "r");
	if (input_file)
	{
		fftw_import_wisdom_from_file(input_file);
		fclose(input_file);
	}

	if (direc == -1) 
		plan = fftw2d_create_plan(anzy, anzx, FFTW_FORWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);
	else
		plan = fftw2d_create_plan(anzy, anzx, FFTW_BACKWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);
	wisdom_string = fftw_export_wisdom_to_string();
	if (wisdom_string != NULL) 
	{
		output_file = fopen("fftw.wisdom", "w");                     
		fftw_export_wisdom_to_file(output_file);
		fclose(output_file);
		fftw_free(wisdom_string);
	} 
	
	fftwnd_threads_one(4, plan, in, NULL);

	fftwnd_destroy_plan(plan);	

}


