/*  ------------------------------------------- */
/*  FFTW2IDL: Wrapper for using FFTW in IDL     */
/*  14.2.2002 by Andreas Reigber                */
/*  published under the GNU public licence (GPL)*/
/*  Email: anderl@nought.de                     */
/*  ------------------------------------------- */
/*  FFTW2D.C                                    */
/*  2D transformation over complex 2D Array     */
/*  ------------------------------------------- */
/*  Usage in IDL:                               */
/*  fftw2d,arr,direc                            */
/*  ------------------------------------------- */

#define FFTW_ENABLE_FLOAT
#include "fftw.h"
//#include "rfftw.h"
#include "export.h"
//#include <stdlib.h>
#include <stdio.h>
#include <windows.h>

#ifdef WIN32
	#define IDL_DLL_API __declspec(dllexport)
#endif

#ifdef WIN32
IDL_DLL_API 
#endif
IDL_LONG fftw2d(int argc,char *argv[])
{
	fftwnd_plan plan;
	FILE *input_file;
	FILE *output_file;
	fftw_complex *in;
	int direc;
	int anzx, anzy;

	if (argc != 4) return -1;

	// parameters
		in = (fftw_complex *)argv[0];
		anzx = *(int *)argv[1];
		anzy = *(int *)argv[2];
		direc = *(int *)argv[3];
	
	if (direc != 1 && direc != -1 )
	{
#ifdef WIN32
		MessageBox(NULL,"ERROR: Direction argument has to be -1 (forward FFT) or 1 (backward FFT)","ERROR",MB_ICONEXCLAMATION);
#else
		printf("ERROR: Direction argument has to be -1 (forward FFT) or 1 (backward FFT)\n");
#endif
		return 0;
	}

	input_file = fopen("fftw.wisdom", "r");
	fftw_import_wisdom_from_file(input_file);
	fclose(input_file);

	if (direc == -1) 
		plan = fftw2d_create_plan(anzy, anzx, FFTW_FORWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);
	else
		plan = fftw2d_create_plan(anzy, anzx, FFTW_BACKWARD,FFTW_MEASURE | FFTW_USE_WISDOM | FFTW_IN_PLACE);

	output_file = fopen("fftw.wisdom", "w");                     
	fftw_export_wisdom_to_file(output_file);
	fclose(output_file);

	fftwnd_one(plan, in, NULL);

	fftwnd_destroy_plan(plan);	

	return 1;
}


